#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-io-server-config v. 2.1.5
#
# Post-installation script for configuring the gLite Computing Element Service
# Alberto Di Meglio  < alberto.di.meglio@cern.ch >
# Leanne Guy  < leanne.guy@cern.ch >
#
# Version info: $Id: glite-io-server-config.py,v 1.44.2.10 2006/03/03 00:04:49 dimeglio Exp $
#
# Usage: python glite-io-server-config [-c|-v|-h|-siteconfig=<site config url>|--start|--stop|--status|--configure|--help]
#        -c                             print configuration
#        -v                             print version
#        -h,--help                      print usage info
#        -siteconfig=<site config url>  use site configuration file
#        --start                        start the service
#        --stop                         start the service
#        --status                       shows the status of the service
#        --configure                    configures the service
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,socket,pwd,time
import sys
import getopt
import commands

from  gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams
from gridSecurity import gridSecurity
from globus import Globus
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params                         # all config values from the XML file
global globus                         # the one and only globus service instance
global rgmaServicetool

class glite_io_server:

    def __init__(self):
        self.verbose = 0
        self.version = "2.1.5"
        self.name = "glite-io-server"
        self.friendly_name = "gLite I/O Server"
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf                         print the service configuration'
        print '    -v, --version                           print the version of the configuration script'
        print '    -h, --help                              print this usage information'
        print '    --start                                 start the service'
        print '    --stop                                  stop the service'
        print '    --status                                print service status (0=running,1=idle)'
        print '    --configure                             configure the io-server service'
        print '    --siteconfig=<site_config_URL>          use site configuration file at URL'
        print '\n'

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    def configure(self):
        
        print "\n"
        error_level = 0
        
        #-------------------------------------------------------------------------------
        # Check for libshift
        #-------------------------------------------------------------------------------

        if self.checkForLibshift():
            return 1
        
        #-------------------------------------------------------------------------------
        # VO list notice
        #-------------------------------------------------------------------------------

        print "\nThis I/O Server will be configured to serve the following VOs:"
        for vo in glib.getVoList():
            print "=> %s" % vo
            
        #-------------------------------------------------------------------------------
        # Update the $GLITE_LOCATION/etc/glite-mkgridmap.conf file if required
        #-------------------------------------------------------------------------------
        
        # Use optional parameter to pass  isgroup = 0, since this is not a pool
        # account, but a single user
        isgroup = 0
        ret_val = gsec.configure_mkgridmap('init.username')
        if ret_val == 1:
            return 1
        elif ret_val == 2:
            error_level = 2

        #-------------------------------------------------------------------------------
        # Installs the Security Utilities
        #--------------------------------------------------------
        
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "\nInstalling gLite Security Utilities               ",
            glib.printFailedMessage()
            return 1

        print "\nInstalling gLite Security Utilities                        ",
        glib.printOkMessage()
            
        #-----------------------------------------------------------------------------------------
        # Create the user account
        #-----------------------------------------------------------------------------------------

        print 'Creating and configuring the %s user account' % self.friendly_name

        if ((params.has_key('init.uid') and not params.has_key('init.gid')) or
            (params.has_key('init.gid') and not params.has_key('init.uid'))):
            print "Both the uid and gid must be set or neither - please set them and rerun"
            return 1

        if not params.has_key('init.uid'):
            params['init.uid'] = "0"
        if not params.has_key('init.gid'):
            params['init.gid'] = "0"

        if params['init.uid'] == "":
            params['init.uid'] = "0"

        if params['init.gid'] == "":
            params['init.gid'] = "0"

        # Create group if doesn't exist, scripts terminates
        # if group already exist with different GID
        if self.verbose:
            print "Verifying existence of group %s" % params['init.groupname']
        if params['init.gid']:
            gid = glib.add_group(params['init.groupname'],params['init.gid'])
        else:
            gid = glib.add_group(params['init.groupname'])
        if self.verbose:
            print 'Group %s has been added or verified to have GID=%d' \
                % (params['init.groupname'], gid)
    
        # Create account and add it to group
        if params['init.uid']:
            (uid,gid) = glib.add_user(params['init.username'],params['init.groupname'], \
                params['init.uid'])
        else:
            (uid,gid) = glib.add_user(params['init.username'],params['init.groupname'])

        print 'User account created                                    ',
        glib.printOkMessage()

        # Check home directory, create if not existing
        if self.verbose:
            print "Checking user home directory %s" % pwd.getpwnam(params['init.username'])[5]
        glib.check_dir("%s" % pwd.getpwnam(params['init.username'])[5],0700, uid, gid)
    
        # Reset environment, just in case
        glib.setUserEnv(params['init.username'])

        # Verify and create service directories
        if self.verbose:
            print "Verify directory permissions"
        glib.check_dir(os.environ['GLITE_LOCATION_VAR'],0755, \
            int(params['init.uid']),int(params['init.gid']))
        glib.check_dir(os.environ['GLITE_LOCATION_LOG'],0755, \
            int(params['init.uid']),int(params['init.gid']))
        glib.check_dir(os.environ['GLITE_LOCATION_TMP'],0755, \
            int(params['init.uid']),int(params['init.gid']))
        glib.check_dir("%s/lock" % os.environ['GLITE_LOCATION_VAR'],0755, \
            int(params['init.uid']),int(params['init.gid']))
        glib.check_dir("%s/tmp" % os.environ['GLITE_LOCATION_VAR'],0755, \
            int(params['init.uid']),int(params['init.gid']))

        #-----------------------------------------------------------------------------------------
        # Copy the host certificates to the user account
        #-----------------------------------------------------------------------------------------
        
        if params['service.certificates.type'] == 'host':
            
            print "Copying the host certificate to the %s user account" % self.friendly_name
    
            uid,gid = glib.get_user_info(params['init.username'])
            
            # Check certificate directory, create if not existing
            if self.verbose:
                print "Checking user certificates directory %s/%s" % (pwd.getpwnam(params['init.username'])[5],\
                    params['user.certificate.path'])
            glib.check_dir("%s/%s" % (pwd.getpwnam(params['init.username'])[5],\
                params['user.certificate.path']),0700, uid, gid)
        
            # Copy certificates
            if self.verbose:
                print "Copying host certificates to I/O Server user certificates directory"
            os.system("cp %s %s/%s/usercert.pem" % (params['host.certificate.file'], \
                pwd.getpwnam(params['init.username'])[5], params['user.certificate.path']))
            os.system("cp %s %s/%s/userkey.pem" % (params['host.key.file'], \
                pwd.getpwnam(params['init.username'])[5], params['user.certificate.path']))
            os.chown("%s/%s/usercert.pem" % (pwd.getpwnam(params['init.username'])[5], \
                params['user.certificate.path']), uid, gid)
            os.chown("%s/%s/userkey.pem" % (pwd.getpwnam(params['init.username'])[5], \
                params['user.certificate.path']), uid, gid)
            os.chmod("%s/%s/usercert.pem" % (pwd.getpwnam(params['init.username'])[5], \
                params['user.certificate.path']),0644)
            os.chmod("%s/%s/userkey.pem" % (pwd.getpwnam(params['init.username'])[5], \
                params['user.certificate.path']),0400)
        
            if (os.path.exists('%s/%s/userkey.pem' % (pwd.getpwnam(params['init.username'])[5],\
                    params['user.certificate.path']))) and \
                    (os.path.exists('%s/%s/usercert.pem' % (pwd.getpwnam(params['init.username'])[5],\
                    params['user.certificate.path']))):
                print "Copied host certificates to %s/%s            " % (pwd.getpwnam(params['init.username'])[5],\
                    params['user.certificate.path']),
                glib.printOkMessage()
            else:
                print "An error occurred while copying the host certificates to %s/%s                " % (pwd.getpwnam(params['init.username'])[5],\
                    params['user.certificate.path']),
                glib.printFailedMessage()
                return 1
    
            # Define service certificates location
            params['service.certificate.file'] = "%s/%s/usercert.pem" % (pwd.getpwnam(params['init.username'])[5],\
                params['user.certificate.path']);
            params['service.key.file'] = "%s/%s/userkey.pem" % (pwd.getpwnam(params['init.username'])[5],\
                params['user.certificate.path']);
                    
        elif params['service.certificates.type'] == 'service':
    
            if (os.path.exists(params['service.certificate.file'])) and \
                    (os.path.exists(params['service.key.file'])):

                # Check permissions
                uid,gid = glib.get_user_info(params['init.username'])
                glib.check_dir("%s" % os.path.dirname(params['service.certificate.file']),0755, uid, gid)
                os.chown(params['service.certificate.file'], uid, gid)
                os.chmod(params['service.certificate.file'],0644)
                glib.check_dir("%s" % os.path.dirname(params['service.key.file']),0755, uid, gid)
                os.chown(params['service.key.file'], uid, gid)
                os.chmod(params['service.key.file'],0400)
    
                print "Using certificates %s and %s            " % (params['service.certificate.file'],\
                    params['service.key.file']),
                glib.printOkMessage()
            else:
                print "[WARNING] Service certificates %s and %s not found" % (params['service.certificate.file'],\
                    params['service.key.file'])
                print "Please install the certificates with correct ownership before starting the %s" % self.friendly_name
                error_level = 2

        else:
            print "The type of certificates to use for this service is not specified. Cannot continue    ",
            glib.printFailedMessage()
            return 1
            
        # SSL configuration
        #glib.export('X509_USER_CERT',params['service.certificate.file']);
        #glib.export('X509_USER_KEY',params['service.key.file']);
                    
        #-----------------------------------------------------------------------------------------
        # Link the gridmap file to the user account
        #-----------------------------------------------------------------------------------------
    
        print "Linking the gridmap file to the user account"
    
        link = "%s/.gridmap" % pwd.getpwnam(params['init.username'])[5]
        if os.path.islink(link):
            os.remove(link)
           
        if not os.path.exists(params['host.gridmapfile']):
            print "The %s doesn't exist. Creating an empty one" % params['host.gridmapfile']
            os.system('/bin/touch %s' % params['host.gridmapfile'])
           
        os.system('/bin/ln -s %s %s' % (params['host.gridmapfile'],link))
        
        if os.path.islink(link):
            print "Linked %s to %s            " % (params['host.gridmapfile'],link),
            glib.printOkMessage()
        
        #-----------------------------------------------------------------------------------------
        # Now configure all instances
        #-----------------------------------------------------------------------------------------

        instanceRank = 0
        instances = glib.getInstances('io-server')
        if len(instances) > 0:
            for instanceName in instances:
                
                print "Processing I/O Server instance %s " % instanceName

                instanceParams = ConfigParams()
                try:
                    instanceParams = glib.getInstanceParams(instanceName)
                except ValueError, msg:
                    glib.printErrorMessage("%s - Could not get parameters. Please check configuration files." % msg)
                    return 1
                except LookupError, msg:
                    glib.printErrorMessage("%s - Could not get parameters. Please check configuration files." % msg)
                    return 1
        
                # Calculate server ports
                if not instanceParams.has_key('io-daemon.Port'):
                    instanceParams['io-daemon.Port'] = '9999'
                if instanceParams['io-daemon.Port'] == '':
                    instanceParams['io-daemon.Port'] = '9999'
                if instanceParams['autocalculate.port'] == 'true':
                    instanceParams['io-daemon.Port'] = int(instanceParams['io-daemon.Port']) - 10*instanceRank
                    instanceRank += 1

                #-----------------------------------------------------------------------------------------
                # Create configuration properties file
                #-----------------------------------------------------------------------------------------
               
                print "Creating %s instance %s service configuration file" % (self.friendly_name, instanceName)
            
                # Select catalog service
                catalog = instanceParams['init.CatalogType']
                if self.verbose:
                    print "Selected catalog type is %s" % catalog
                
                # Temporary properties file for the configuration
                properties_file = "%s/etc/config/glite-io-server-%s.properties" \
                    % (os.environ['GLITE_LOCATION'],instanceParams['instanceDescription'])
                f = open(properties_file,'w')
        
                # All catalogs parameters
                if self.verbose:
                    print "Writing common catalog parameters"
        
                # io-daemon parameters
                for param in instanceParams:
                    if param.find('io-daemon') != -1:
                        f.write('%s = %s\n' % (param,instanceParams[param]))
        
                # io-resolve-common parameters
                for param in instanceParams:
                    if param.find('io-resolve-common') != -1:
                        f.write('%s = %s\n' % (param,instanceParams[param]))
        
                # EDG Replica and Metadata Catalog parameters
                if catalog == "catalogs":
                    if self.verbose:
                        print "Writing EDG catalog and metadata parameters"
        
                    # io-authz-catalog parameters
                    for param in instanceParams:
                        if param.find('io-authz-catalog') != -1:
                            f.write('%s = %s\n' % (param,instanceParams[param]))
        
                    # io-resolve-catalogs parameters
                    # The value is the same for these two parameters, we ask only one in the config file
                    if not instanceParams.has_key('io-resolve-catalogs.RmcEndpoint'):
                        instanceParams['io-resolve-catalogs.RmcEndpoint'] = instanceParams['io-authz-catalogs.RmcEndpoint']
                    for param in instanceParams:
                        if param.find('io-resolve-catalogs') != -1:
                            f.write('%s = %s\n' % (param,instanceParams[param]))
                                
                # Fireman Catalog parameters
                if catalog == "fireman":
                    if self.verbose:
                        print "Writing Fireman catalog parameters"
        
                    # io-authz-fas parameters
                    for param in instanceParams:
                        if param.find('io-authz-fas') != -1:
                            f.write('%s = %s\n' % (param,instanceParams[param]))
        
                    # io-resolve-fireman parameters
                    for param in instanceParams:
                        if param.find('io-resolve-fireman') != -1:
                            f.write('%s = %s\n' % (param,instanceParams[param]))
        
                # File and Replica Catalog parameters
                if catalog == "fr":
                    if self.verbose:
                        print "Writing File and Replica catalog parameters"
        
                    # io-authz-fas parameters
                    for param in instanceParams:
                        if param.find('io-authz-fas') != -1:
                            f.write('%s = %s\n' % (param,instanceParams[param]))
        
                    # io-resolve-fr parameters
                    for param in instanceParams:
                        if param.find('io-resolve-fr') != -1:
                            f.write('%s = %s\n' % (param,instanceParams[param]))
        
                # Alien Catalog parameters
                if catalog == "alien":
                    if self.verbose:
                        print "Writing Alien catalog parameters"
        
                    # io-authz-alien parameters
                    for param in instanceParams:
                        if param.find('io-authz-alien') != -1:
                            f.write('%s = %s\n' % (param,instanceParams[param]))
        
                    # io-resolve-alien parameters
                    for param in instanceParams:
                        if param.find('io-resolve-alien') != -1:
                            f.write('%s = %s\n' % (param,instanceParams[param]))
        
                # Log properties for io client
                if self.verbose:
                    print "Writing logging parameters"
                f.write('log.Priority = %s\n' % instanceParams['log.Priority'])
                f.write('log.FileName = %s\n' % instanceParams['log.FileName'])
                  
                f.close()
        
                #-----------------------------------------------------------------------------------------
                # Run the configuration tool
                #-----------------------------------------------------------------------------------------
               
                # Run the configuration.
                # Create the files locally and then copy to $GLITE_LOCATION/etc and
                # change the ownership
                    
                if self.verbose:
                    print "Running the configuration tool: glite_data_config_generator"
        
                uid = pwd.getpwnam(instanceParams['init.username'])[2]
                gid = pwd.getpwnam(instanceParams['init.username'])[3]
                os.chown('%s/etc' % os.environ['GLITE_LOCATION'],uid,gid)
                try:
                    config_generator = '%s/bin/glite_data_config_generator' % os.environ['GLITE_LOCATION']
                    template = '%s/etc/config/templates/glite-io-server-%s.service.config.xml' \
                        % (os.environ['GLITE_LOCATION'],catalog)
                    output = '%s/etc/glite-io-server-%s.properties.xml' \
                        % (os.environ['GLITE_LOCATION'],instanceParams['instanceDescription'])
                    log = '%s/etc/glite-io-server-%s.log-properties' \
                        % (os.environ['GLITE_LOCATION'],instanceParams['instanceDescription'])
                
                    cmd = '/bin/su -l %s -c "%s -f %s -o %s -l %s  -p %s"'  \
                          % (instanceParams['init.username'],config_generator,template,output,log,properties_file)
                    if self.verbose:
                        print cmd
                    if os.path.exists(output):
                        os.remove(output)
                    os.system(cmd)
                except:
                    print "An exception occurred while creating the service configuration files for instance %s" \
                        % instanceName
                    # Set the permissions back
                    os.chown('%s/etc' % os.environ['GLITE_LOCATION'],0,0)
                    return 1
                else:
                    # Set the permissions back
                    os.chown('%s/etc' % os.environ['GLITE_LOCATION'],0,0)
            
                if os.path.exists(output) and os.path.exists(log):
                    print "Service configuration files successfully created in %s/etc" % os.environ['GLITE_LOCATION']
                else:
                    return 1
            
        #-------------------------------------------------------------------
        # RGMA servicetool: configure servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
            
            # Instantiate the rgma-servicetool class
            rgmaServicetool = gliteRgmaServicetool()
            rgmaServicetool.verbose = self.verbose
            
            for instance in glib.getInstances('io-server'):

                ioParams = glib.getInstanceParams(instance)
                
                # Create Local Logger instance
                serviceId = "%s_%s_%s" % (glib.fq_hostname, ioParams['instanceDescription'], params['gliteio.serviceType'])
                servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
                
                # set params
                servicetoolInstance.setServiceName('gLite I/O Server (%s) at %s' \
                    % (ioParams['instanceDescription'],glib.fq_hostname))
                servicetoolInstance.setServiceType(params['gliteio.serviceType'])
                servicetoolInstance.setServiceVersion(self.version)
                servicetoolInstance.setStatusScript('%s/bin/glite_data_config_service glite-io-server-%s status' % (os.environ['GLITE_LOCATION'],ioParams['instanceDescription']))
                servicetoolInstance.setEndpoint('gliteio://%s:%s' % (glib.fq_hostname,ioParams['io-daemon.Port']))
                
                # add instance to the gLite configuration
                if servicetoolInstance.add() == 1:
                    return 1
                
            # Configure servicetool
            if rgmaServicetool.configure(glib):
                # error in configuring servicetool
                return 1
            
        return error_level
        
    #-------------------------------------------------------------------------------
    # Start all services
    #-------------------------------------------------------------------------------

    def start(self):
        
        if self.checkForLibshift():
            return 1

#        print "Starting rfiod .... "
#        pid = glib.getPID('rfiod')
#        if pid != 0:
#            print "rfiod running process detected, pid = %s " %  pid
#            os.system("/sbin/service rfiod stop")
#        os.system("/sbin/service rfiod start")
#
#        pid = glib.getPID('rfiod')
#        if pid == 0:
#            return 1

        instances = glib.getInstances('io-server')
        if len(instances) > 0:
            for instanceName in instances:

                instanceParams = {}
                try:
                    instanceParams = glib.getInstanceParams(instanceName)
                except ValueError, msg:
                    glib.printErrorMessage("%s - Could not get parameters. Please check configuration files." % msg)
                    return 1
                except LookupError, msg:
                    glib.printErrorMessage("%s - Could not get parameters. Please check configuration files." % msg)
                    return 1
        
                # Check that service have started
                catalog = instanceParams['init.CatalogType']
                pid = glib.getPID('glite-io-server-%s-%s' % (instanceParams['vo.name'],catalog))
                print "\nStarting the %s %s instance...." % (self.friendly_name, instanceName)
                uid,gid = glib.get_user_info(params['init.username'])
                os.system("/bin/touch %s" % instanceParams['log.FileName'])
                os.chown(instanceParams['log.FileName'],uid,gid)
                if os.path.exists("%s/tmp/glite-io-server-%s-%s.pid" \
                  % (os.environ['GLITE_LOCATION_VAR'],instanceParams['vo.name'],catalog)):
                    ios_pid = string.rstrip(open("%s/tmp/glite-io-server-%s-%s.pid" \
                      % (os.environ['GLITE_LOCATION_VAR'],instanceParams['vo.name'],catalog),'r').readline())
                    print "glite-io-server-%s-%s running process detected, pid = %s " % (instanceParams['vo.name'],catalog,ios_pid)
                    print "Stopping running glite-io-server-%s-%s process" % (instanceParams['vo.name'],catalog)
                    cmd = '/bin/su -l %s -c "%s/bin/glite_data_config_service glite-io-server-%s-%s stop"' \
                      % (instanceParams['init.username'], os.environ['GLITE_LOCATION'],instanceParams['vo.name'],catalog)
                    os.system(cmd)
                
                cmd = '/bin/su -l %s -c "%s/bin/glite_data_config_service glite-io-server-%s-%s start"' \
                      % (instanceParams['init.username'], os.environ['GLITE_LOCATION'],instanceParams['vo.name'],catalog)
                if self.verbose:
                    print cmd
                os.system(cmd)
                print '\n'
                        # Check that service have started
                pid = glib.getPID('glite-io-server-%s-%s' % (instanceParams['vo.name'],catalog))
                if pid == 0:
                    return 1
        
        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
                
            errorcode = rgmaServicetool.start()
            if (errorcode != 0):
                return 1
            
        return 0
        
    #-------------------------------------------------------------------------------
    # Stop all services
    #-------------------------------------------------------------------------------

    def stop(self):

        error_level = 0

#        print "Stopping rfiod .... "
#        os.system("/sbin/service rfiod stop")
#
#        # Check if process has stopped
#        pid = glib.getPID('rfiod')
#        if pid != 0:
#            glib.printFailedMessage()

        instances = glib.getInstances('io-server')
        if len(instances) > 0:
            for instanceName in instances:

                instanceParams = {}
                try:
                    instanceParams = glib.getInstanceParams(instanceName)
                except ValueError, msg:
                    glib.printErrorMessage("%s - Could not get parameters. Please check configuration files." % msg)
                    error_level = 1
                except LookupError, msg:
                    glib.printErrorMessage("%s - Could not get parameters. Please check configuration files." % msg)
                    error_level = 1
        
                catalog = instanceParams['init.CatalogType']

                print "\nStopping the %s %s instance..." % (self.friendly_name, instanceName)
                if os.path.exists("%s/tmp/glite-io-server-%s-%s.pid" \
                  % (os.environ['GLITE_LOCATION_VAR'],instanceParams['vo.name'],catalog)):
                    ios_pid = string.rstrip(open("%s/tmp/glite-io-server-%s-%s.pid" \
                      % (os.environ['GLITE_LOCATION_VAR'],instanceParams['vo.name'],catalog),'r').readline())
                    cmd = '/bin/su -l %s -c "%s/bin/glite_data_config_service glite-io-server-%s-%s stop"' \
                      % (instanceParams['init.username'], os.environ['GLITE_LOCATION'],instanceParams['vo.name'],catalog)
                    os.system(cmd)
                else:
                    glib.printInfoMessage("glite-io-server-%s-%s was already stopped" % (instanceParams['vo.name'],catalog))
                
                # Check if process has stopped
                pid = glib.getPID('glite-io-server-%s-%s ' % (instanceParams['vo.name'],catalog))
                if pid != 0:
                    error_level = 1
                    glib.printFailedMessage()

        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
            
            if rgmaServicetool.stop():
                error_level = 1
    
        return error_level
        
    #-------------------------------------------------------------------------------
    # Service status
    #-------------------------------------------------------------------------------

    def status(self):

        error_level = 0
        
#        print "Checking rfiod .... "
#        if os.system("/sbin/service rfiod status"):
#            error_level = 1
        
        instances = glib.getInstances('io-server')
        if len(instances) > 0:
            for instanceName in instances:

                instanceParams = {}
                try:
                    instanceParams = glib.getInstanceParams(instanceName)
                except ValueError, msg:
                    glib.printErrorMessage("%s - Could not get parameters. Please check configuration files." % msg)
                    return 1
                except LookupError, msg:
                    glib.printErrorMessage("%s - Could not get parameters. Please check configuration files." % msg)
                    return 1
        
                catalog = instanceParams['init.CatalogType']

                print "\nChecking the %s %s instance..." % (self.friendly_name, instanceName)
                cmd = '/bin/su -l %s -c "%s/bin/glite_data_config_service glite-io-server-%s-%s status"' \
                      % (instanceParams['init.username'], os.environ['GLITE_LOCATION'], instanceParams['vo.name'], catalog)
                if os.system(cmd):
                    error_level = 1

        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
            
            if rgmaServicetool.status() != 0:
                error_level = 1
    
        return error_level

    #-------------------------------------------------------------------------------
    # Check for libshift
    #-------------------------------------------------------------------------------
    def checkForLibshift(self):
        
        # Check if dpm is installed
        if(os.path.exists("/opt/lcg/lib/libdpm.so")):
            os.system("ln -fs /opt/lcg/lib/libdpm.so /opt/lcg/lib/libshift.so")
            os.system("ldconfig")
            
        if not os.path.exists('/opt/lcg/lib/libshift.so') and \
           not os.path.exists('/usr/lib/libshift.so'):
            glib.printErrorMessage('Library libshift.so not found')
            glib.printErrorMessage('Please install a supported SE client before continuing')

            return 1
            
        return 0
    
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    
    params['gliteio.serviceType'] = 'org.glite.GliteIO'
    
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():

    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    glib.export('GLOBUS_LOCATION');

    # Fix X509_USER_{CERT|KEY} problem in setenv file
    if os.path.exists(params['installer.export.filename']):
        infile = open(params['installer.export.filename'],'r')
        outfile = open(params['installer.export.filename']+'.1','w')
        for line in infile:
            if line.find('X509_USER_CERT') == -1 and \
                line.find('X509_USER_KEY') == -1:
                outfile.write(line)

        outfile.close()
        infile.close()
        os.rename(params['installer.export.filename']+'.1',params['installer.export.filename'])

    glib.export('JAVA_HOME')

    # bin and lib paths
    glib.addEnvPath("PATH","%s/bin:%s/externals/bin:%s/bin:%s/bin" \
        % (os.environ['GLOBUS_LOCATION'], os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'], os.environ['JAVA_HOME']))
    glib.addEnvPath("LD_LIBRARY_PATH","/usr/local/lib:%s/lib:%s/externals/lib:%s/lib" % (os.environ['GLOBUS_LOCATION'],os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION']))

    # Set environment
    glib.setUserEnv()
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid() == 0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    
    # Load parameters
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration(["%s/../glite-io-server.cfg.xml" % glib.getScriptPath(), \
            "%s/../glite-security-utils.cfg.xml" % glib.getScriptPath(), \
            "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()],params):
        print "An error occurred while configuring the service"
        sys.exit(1)
    
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    #Instantiate grdSecurity classes
    gsec = gridSecurity(glib)
    gsec.setVerbosity(verbose)
    gsec.params = params

    # Instantiate the service classes
    service = glite_io_server()
    service.verbose = verbose

    globus = Globus(params)

    # Instantiate the rgma servicetool class
    rgmaServicetool = gliteRgmaServicetool()
    rgmaServicetool.verbose = verbose
    
    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'start', 'stop', 'status', 'configure', 'siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # Check cli options
    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)

        if o == "--configure":

            service.copyright()
            service.showVersion()
            service.banner()
            
            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s service" \
                            % service.friendly_name
                        sys.exit(1)    
                
            # Display the parameters if verbose output
            if verbose: 
                glib.print_params(params)
                
            # Stop all services
            glib.printInfoMessage("\n\nStopping all running I/O Server services...")
            service.stop()
            
            # Configure the service
            return_result = service.configure()

            if return_result == 0:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running I/O Server services...")
                service.stop()
                
                glib.registerService()

                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                print "You can now start the service using the --start option of this script\n\n"

                sys.exit(0)

            elif return_result == 2:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running I/O Server services...")
                service.stop()
                
                print "\n\nThe %s configuration was completed,\n" % service.friendly_name
                print "but warnings were issued. Please revise them and re-run the script\n"
                print "or configure I/O Server manually\n"

                sys.exit(2)
            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name

                sys.exit(1)
            
        if o == "--start":
            if service.start() == 0:
                print "\n\nThe %s was successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while starting the %s" \
                    % service.friendly_name
                sys.exit(1)
                
        if o == "--stop":
            if service.stop() == 0:
                print "\n\nThe %s was successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while stopping the %s" \
                    % service.friendly_name
                sys.exit(1)
                
        if o == "--status":
            retval = service.status()
            sys.exit(retval)
